/************************************************************************/
/*                                                                      */
/*  Dc_Shared.c : Module pour la bibliothque de fonctions gnriques.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2013  */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#if defined(WIN32) || defined(WIN64)
#include <malloc.h>
#include <io.h>
#endif

#include "Dc_Shared.h"

/*************************************************************/
/*  LoadFileData() :  Rcupration des donnes d'un fichier. */
/*************************************************************/
unsigned char *LoadFileData(char *file_path, int *data_length_rtn)
{
  FILE *fd;
  int nb_read, file_size;
  unsigned char *data;

  /* Ouverture du fichier */
#if defined(WIN32) || defined(WIN64)  
  fd = fopen(file_path,"rb");
#else
  fd = fopen(file_path,"r");
#endif  
  if(fd == NULL)
    return(NULL);

  /* Taille du fichier */
  fseek(fd,0L,SEEK_END);
  file_size = ftell(fd);
  fseek(fd,0L,SEEK_SET);
   
  /* Allocation mmoire */
  data = (unsigned char *) calloc(1,file_size+1);
  if(data == NULL)
    {
      fclose(fd);
      return(NULL);
    }

  /* Lecture des donnes */
  nb_read = (int) fread(data,1,file_size,fd);
  if(nb_read < 0)
    {
      free(data);
      fclose(fd);
      return(NULL);
    }
  data[nb_read] = '\0';

  /* Fermeture du fichier */
  fclose(fd);

  /* Renvoi les donnes et la taille */
  *data_length_rtn = nb_read;
  return(data);
}


/********************************************************************/
/*  CreateBinaryFile() :  Cration d'un fichier binaire sur disque. */
/********************************************************************/
int CreateBinaryFile(char *file_path, int data_length, unsigned char *data)
{
  FILE *fd;
  int nb_write;

  /* Ouverture du fichier */
#if defined(WIN32) || defined(WIN64)  
  fd = fopen(file_path,"wb");
#else
  fd = fopen(file_path,"w");
#endif
  if(fd == NULL)
    return(1);

  /* Lecture des donnes */
  nb_write = (int) fwrite(data,1,data_length,fd);

  /* Fermeture du fichier */
  fclose(fd);

  /* OK */
  if(nb_write != data_length)
    return(2);
  return(0);
}


/************************************************************************/
/*  my_stricmp() :  On compare les chaines de caractere sans la casse.  */
/************************************************************************/
int my_stricmp(char *string1, char *string2)
{
#if defined(WIN64) | defined(WIN32) 
  return(stricmp(string1,string2));
#else
  char a, b;
  
  while(*string1)
    {
      a = toupper(*string1);
      b = toupper(*string2);
      if(a != b)
        break;
      string1++;
      string2++;
    }
  if(*string1 == 0)
    return(0);
    
  return((*string1 - *string2) > 0 ? 1:-1); 
#endif
}

/************************************************************************/
